/* original parser id follows */
/* yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93" */
/* (use YYMAJOR/YYMINOR for ifdefs dependent on parser version) */

#define YYBYACC 1
#define YYMAJOR 2
#define YYMINOR 0
#define YYPATCH 20200910

#define YYEMPTY        (-1)
#define yyclearin      (yychar = YYEMPTY)
#define yyerrok        (yyerrflag = 0)
#define YYRECOVERING() (yyerrflag != 0)
#define YYENOMEM       (-2)
#define YYEOF          0
#define YYPREFIX "yy"

#define YYPURE 0

#line 15 "../../lib/common/htmlparse.y"

#include "render.h"
#include "htmltable.h"
#include "htmllex.h"

extern int yyparse(void);

typedef struct sfont_t {
    textfont_t *cfont;	
    struct sfont_t *pfont;
} sfont_t;

static struct {
  htmllabel_t* lbl;       /* Generated label */
  htmltbl_t*   tblstack;  /* Stack of tables maintained during parsing */
  Dt_t*        fitemList; /* Dictionary for font text items */
  Dt_t*        fspanList; 
  agxbuf*      str;       /* Buffer for text */
  sfont_t*     fontstack;
  GVC_t*       gvc;
} HTMLstate;

/* free_ritem:
 * Free row. This closes and frees row's list, then
 * the pitem itself is freed.
 */
static void
free_ritem(Dt_t* d, pitem* p,Dtdisc_t* ds)
{
  dtclose (p->u.rp);
  free (p);
}

/* free_item:
 * Generic Dt free. Only frees container, assuming contents
 * have been copied elsewhere.
 */
static void
free_item(Dt_t* d, void* p,Dtdisc_t* ds)
{
  free (p);
}

/* cleanTbl:
 * Clean up table if error in parsing.
 */
static void
cleanTbl (htmltbl_t* tp)
{
  dtclose (tp->u.p.rows);
  free_html_data (&tp->data);
  free (tp);
}

/* cleanCell:
 * Clean up cell if error in parsing.
 */
static void
cleanCell (htmlcell_t* cp)
{
  if (cp->child.kind == HTML_TBL) cleanTbl (cp->child.u.tbl);
  else if (cp->child.kind == HTML_TEXT) free_html_text (cp->child.u.txt);
  free_html_data (&cp->data);
  free (cp);
}

/* free_citem:
 * Free cell item during parsing. This frees cell and pitem.
 */
static void
free_citem(Dt_t* d, pitem* p,Dtdisc_t* ds)
{
  cleanCell (p->u.cp);
  free (p);
}

static Dtdisc_t rowDisc = {
    offsetof(pitem,u),
    sizeof(void*),
    offsetof(pitem,link),
    NIL(Dtmake_f),
    (Dtfree_f)free_ritem,
    NIL(Dtcompar_f),
    NIL(Dthash_f),
    NIL(Dtmemory_f),
    NIL(Dtevent_f)
};
static Dtdisc_t cellDisc = {
    offsetof(pitem,u),
    sizeof(void*),
    offsetof(pitem,link),
    NIL(Dtmake_f),
    (Dtfree_f)free_item,
    NIL(Dtcompar_f),
    NIL(Dthash_f),
    NIL(Dtmemory_f),
    NIL(Dtevent_f)
};

typedef struct {
    Dtlink_t    link;
    textspan_t  ti;
} fitem;

typedef struct {
    Dtlink_t     link;
    htextspan_t  lp;
} fspan;

static void 
free_fitem(Dt_t* d, fitem* p, Dtdisc_t* ds)
{
    if (p->ti.str)
	free (p->ti.str);
    free (p);
}

static void 
free_fspan(Dt_t* d, fspan* p, Dtdisc_t* ds)
{
    textspan_t* ti;

    if (p->lp.nitems) {
	int i;
	ti = p->lp.items;
	for (i = 0; i < p->lp.nitems; i++) {
	    if (ti->str) free (ti->str);
	    ti++;
	}
	free (p->lp.items);
    }
    free (p);
}

static Dtdisc_t fstrDisc = {
    0,
    0,
    offsetof(fitem,link),
    NIL(Dtmake_f),
    (Dtfree_f)free_item,
    NIL(Dtcompar_f),
    NIL(Dthash_f),
    NIL(Dtmemory_f),
    NIL(Dtevent_f)
};


static Dtdisc_t fspanDisc = {
    0,
    0,
    offsetof(fspan,link),
    NIL(Dtmake_f),
    (Dtfree_f)free_item,
    NIL(Dtcompar_f),
    NIL(Dthash_f),
    NIL(Dtmemory_f),
    NIL(Dtevent_f)
};

/* appendFItemList:
 * Append a new fitem to the list.
 */
static void
appendFItemList (agxbuf *ag)
{
    fitem *fi = NEW(fitem);

    fi->ti.str = strdup(agxbuse(ag));
    fi->ti.font = HTMLstate.fontstack->cfont;
    dtinsert(HTMLstate.fitemList, fi);
}	

/* appendFLineList:
 */
static void 
appendFLineList (int v)
{
    int cnt;
    fspan *ln = NEW(fspan);
    fitem *fi;
    Dt_t *ilist = HTMLstate.fitemList;

    cnt = dtsize(ilist);
    ln->lp.just = v;
    if (cnt) {
        int i = 0;
	ln->lp.nitems = cnt;
	ln->lp.items = N_NEW(cnt, textspan_t);

	fi = (fitem*)dtflatten(ilist);
	for (; fi; fi = (fitem*)dtlink(fitemList,(Dtlink_t*)fi)) {
		/* NOTE: When fitemList is closed, it uses free_item, which only frees the container,
		 * not the contents, so this copy is safe.
		 */
	    ln->lp.items[i] = fi->ti;  
	    i++;
	}
    }
    else {
	ln->lp.items = NEW(textspan_t);
	ln->lp.nitems = 1;
	ln->lp.items[0].str = strdup("");
	ln->lp.items[0].font = HTMLstate.fontstack->cfont;
    }

    dtclear(ilist);

    dtinsert(HTMLstate.fspanList, ln);
}

static htmltxt_t*
mkText(void)
{
    int cnt;
    Dt_t * ispan = HTMLstate.fspanList;
    fspan *fl ;
    htmltxt_t *hft = NEW(htmltxt_t);
    
    if (dtsize (HTMLstate.fitemList)) 
	appendFLineList (UNSET_ALIGN);

    cnt = dtsize(ispan);
    hft->nspans = cnt;
    	
    if (cnt) {
	int i = 0;
	hft->spans = N_NEW(cnt,htextspan_t);	
    	for(fl=(fspan *)dtfirst(ispan); fl; fl=(fspan *)dtnext(ispan,fl)) {
    	    hft->spans[i] = fl->lp;
    	    i++;
    	}
    }
    
    dtclear(ispan);

    return hft;
}

static pitem* lastRow (void)
{
  htmltbl_t* tbl = HTMLstate.tblstack;
  pitem*     sp = dtlast (tbl->u.p.rows);
  return sp;
}

/* addRow:
 * Add new cell row to current table.
 */
static pitem* addRow (void)
{
  Dt_t*      dp = dtopen(&cellDisc, Dtqueue);
  htmltbl_t* tbl = HTMLstate.tblstack;
  pitem*     sp = NEW(pitem);
  sp->u.rp = dp;
  if (tbl->flags & HTML_HRULE)
    sp->ruled = 1;
  dtinsert (tbl->u.p.rows, sp);
  return sp;
}

/* setCell:
 * Set cell body and type and attach to row
 */
static void setCell (htmlcell_t* cp, void* obj, int kind)
{
  pitem*     sp = NEW(pitem);
  htmltbl_t* tbl = HTMLstate.tblstack;
  pitem*     rp = (pitem*)dtlast (tbl->u.p.rows);
  Dt_t*      row = rp->u.rp;
  sp->u.cp = cp;
  dtinsert (row, sp);
  cp->child.kind = kind;
  if (tbl->flags & HTML_VRULE)
    cp->ruled = HTML_VRULE;
  
  if(kind == HTML_TEXT)
  	cp->child.u.txt = (htmltxt_t*)obj;
  else if (kind == HTML_IMAGE)
    cp->child.u.img = (htmlimg_t*)obj;
  else
    cp->child.u.tbl = (htmltbl_t*)obj;
}

/* mkLabel:
 * Create label, given body and type.
 */
static htmllabel_t* mkLabel (void* obj, int kind)
{
  htmllabel_t* lp = NEW(htmllabel_t);

  lp->kind = kind;
  if (kind == HTML_TEXT)
    lp->u.txt = (htmltxt_t*)obj;
  else
    lp->u.tbl = (htmltbl_t*)obj;
  return lp;
}

/* freeFontstack:
 * Free all stack items but the last, which is
 * put on artificially during in parseHTML.
 */
static void
freeFontstack(void)
{
    sfont_t* s;
    sfont_t* next;

    for (s = HTMLstate.fontstack; (next = s->pfont); s = next) {
	free(s);
    }
}

/* cleanup:
 * Called on error. Frees resources allocated during parsing.
 * This includes a label, plus a walk down the stack of
 * tables. Note that we use the free_citem function to actually
 * free cells.
 */
static void cleanup (void)
{
  htmltbl_t* tp = HTMLstate.tblstack;
  htmltbl_t* next;

  if (HTMLstate.lbl) {
    free_html_label (HTMLstate.lbl,1);
    HTMLstate.lbl = NULL;
  }
  cellDisc.freef = (Dtfree_f)free_citem;
  while (tp) {
    next = tp->u.p.prev;
    cleanTbl (tp);
    tp = next;
  }
  cellDisc.freef = (Dtfree_f)free_item;

  fstrDisc.freef = (Dtfree_f)free_fitem;
  dtclear (HTMLstate.fitemList);
  fstrDisc.freef = (Dtfree_f)free_item;

  fspanDisc.freef = (Dtfree_f)free_fspan;
  dtclear (HTMLstate.fspanList);
  fspanDisc.freef = (Dtfree_f)free_item;

  freeFontstack();
}

/* nonSpace:
 * Return 1 if s contains a non-space character.
 */
static int nonSpace (char* s)
{
  char   c;

  while ((c = *s++)) {
    if (c != ' ') return 1;
  }
  return 0;
}

/* pushFont:
 * Fonts are allocated in the lexer.
 */
static void
pushFont (textfont_t *fp)
{
    sfont_t *ft = NEW(sfont_t);
    textfont_t* curfont = HTMLstate.fontstack->cfont;
    textfont_t  f = *fp;

    if (curfont) {
	if (!f.color && curfont->color)
	    f.color = curfont->color;
	if ((f.size < 0.0) && (curfont->size >= 0.0))
	    f.size = curfont->size;
	if (!f.name && curfont->name)
	    f.name = curfont->name;
	if (curfont->flags)
	    f.flags |= curfont->flags;
    }

    ft->cfont = dtinsert(HTMLstate.gvc->textfont_dt, &f);
    ft->pfont = HTMLstate.fontstack;
    HTMLstate.fontstack = ft;
}

/* popFont:
 */
static void 
popFont (void)
{
    sfont_t* curfont = HTMLstate.fontstack;
    sfont_t* prevfont = curfont->pfont;

    free (curfont);
    HTMLstate.fontstack = prevfont;
}

#ifdef YYSTYPE
#undef  YYSTYPE_IS_DECLARED
#define YYSTYPE_IS_DECLARED 1
#endif
#ifndef YYSTYPE_IS_DECLARED
#define YYSTYPE_IS_DECLARED 1
#line 415 "../../lib/common/htmlparse.y"
typedef union  {
  int    i;
  htmltxt_t*  txt;
  htmlcell_t*  cell;
  htmltbl_t*   tbl;
  textfont_t*  font;
  htmlimg_t*   img;
  pitem*       p;
} YYSTYPE;
#endif /* !YYSTYPE_IS_DECLARED */
#line 437 "y.tab.c"

/* compatibility with bison */
#ifdef YYPARSE_PARAM
/* compatibility with FreeBSD */
# ifdef YYPARSE_PARAM_TYPE
#  define YYPARSE_DECL() yyparse(YYPARSE_PARAM_TYPE YYPARSE_PARAM)
# else
#  define YYPARSE_DECL() yyparse(void *YYPARSE_PARAM)
# endif
#else
# define YYPARSE_DECL() yyparse(void)
#endif

/* Parameters sent to lex. */
#ifdef YYLEX_PARAM
# define YYLEX_DECL() yylex(void *YYLEX_PARAM)
# define YYLEX yylex(YYLEX_PARAM)
#else
# define YYLEX_DECL() yylex(void)
# define YYLEX yylex()
#endif

#if !(defined(yylex) || defined(YYSTATE))
int YYLEX_DECL();
#endif

/* Parameters sent to yyerror. */
#ifndef YYERROR_DECL
#define YYERROR_DECL() yyerror(const char *s)
#endif
#ifndef YYERROR_CALL
#define YYERROR_CALL(msg) yyerror(msg)
#endif

extern int YYPARSE_DECL();

#define T_end_br 257
#define T_end_img 258
#define T_row 259
#define T_end_row 260
#define T_html 261
#define T_end_html 262
#define T_end_table 263
#define T_end_cell 264
#define T_end_font 265
#define T_string 266
#define T_error 267
#define T_n_italic 268
#define T_n_bold 269
#define T_n_underline 270
#define T_n_overline 271
#define T_n_sup 272
#define T_n_sub 273
#define T_n_s 274
#define T_HR 275
#define T_hr 276
#define T_end_hr 277
#define T_VR 278
#define T_vr 279
#define T_end_vr 280
#define T_BR 281
#define T_br 282
#define T_IMG 283
#define T_img 284
#define T_table 285
#define T_cell 286
#define T_font 287
#define T_italic 288
#define T_bold 289
#define T_underline 290
#define T_overline 291
#define T_sup 292
#define T_sub 293
#define T_s 294
#define YYERRCODE 256
typedef short YYINT;
static const YYINT yylhs[] = {                           -1,
    0,    0,    0,    1,   10,   10,   11,   11,   11,   11,
   11,   11,   11,   11,   11,   11,   13,   14,   15,   16,
   21,   22,   27,   28,   17,   18,   19,   20,   23,   24,
   25,   26,    4,    4,   12,   12,   30,    5,    6,    6,
    6,    6,    6,    6,   29,   29,    9,    9,    9,   32,
    8,    3,    3,    3,   34,    2,   35,    2,   36,    2,
   37,    2,    7,    7,   31,   31,   33,   33,
};
static const YYINT yylen[] = {                            2,
    3,    3,    1,    1,    2,    1,    1,    1,    3,    3,
    3,    3,    3,    3,    3,    3,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    2,    1,    1,    2,    0,    6,    1,    3,
    3,    3,    3,    3,    1,    0,    1,    2,    3,    0,
    4,    1,    2,    3,    0,    4,    0,    4,    0,    4,
    0,    3,    2,    1,    2,    1,    2,    1,
};
static const YYINT yydefred[] = {                         0,
    3,    0,    0,   35,   34,    0,   17,   19,   21,   25,
   27,   29,   31,   23,    0,    8,   39,    0,    0,    6,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   33,    1,    2,    5,    0,    0,    0,    0,    0,    0,
   36,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,   37,   18,   40,    9,   20,   41,
   10,   26,   42,   11,   28,   43,   12,   22,   44,   13,
   30,   14,   32,   15,   24,   16,    0,   50,   47,    0,
    0,    0,   66,    0,   48,    0,    0,   52,    0,    0,
   38,   65,   49,   64,    0,   57,   55,   59,    0,   51,
   68,    0,   53,    0,   63,    0,    0,    0,   62,   67,
   54,   58,   56,   60,
};
static const YYINT yydgoto[] = {                          3,
   15,   88,   89,   16,   17,   18,   98,   79,   80,   19,
   20,   35,   36,   57,   37,   60,   38,   63,   39,   66,
   40,   69,   27,   72,   28,   74,   29,   76,   30,   77,
   86,   81,  104,  107,  106,  108,   99,
};
static const YYINT yysindex[] = {                      -242,
    0,   31,    0,    0,    0, -230,    0,    0,    0,    0,
    0,    0,    0,    0, -231,    0,    0, -209,   31,    0,
 -212,   31,   31,   31,   31,   31,   31,   31,   31, -243,
    0,    0,    0,    0, -212,   31,   31,   31,   31,   31,
    0, -204, -173, -197, -127, -198, -113, -195,  -84, -196,
  -70,  -41,  -12,   17,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, -185,    0,    0, -220,
 -207, -179,    0, -181,    0, -185, -156,    0, -227, -212,
    0,    0,    0,    0, -161,    0,    0,    0, -165,    0,
    0, -169,    0, -207,    0, -152, -151, -142,    0,    0,
    0,    0,    0,    0,
};
static const YYINT yyrindex[] = {                         0,
    0, -162,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, -202,    0,
 -187, -162, -162, -162, -162, -162,    0,    0,    0,    0,
    0,    0,    0,    0, -224,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, -253,    0,    0,    0,    0, -256,    0,    0, -122,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,
};
static const YYINT yygindex[] = {                         0,
   37,  -79,    0,    0,  166,   42,    0,  -73,    0,  304,
  -17,   -2,   -1,   87,    1,   99,    2,   98,    3,  101,
    4,  100,    0,    0,    0,    0,    0,    0,   70,    0,
    0,    0,    0,    0,    0,    0,    0,
};
#define YYTABLESIZE 344
static const YYINT yytable[] = {                         21,
   22,   34,   23,   24,   25,   26,   85,   61,   46,  103,
   46,   46,   93,    1,   46,   46,   46,   46,    2,   21,
   21,   21,   21,   21,  111,   34,   31,   34,   46,   34,
   32,   34,  100,   34,   34,   34,   34,    7,   78,    7,
    7,   55,   82,    7,    7,    7,    7,    7,    7,    7,
  101,  102,   33,   41,   83,   84,    7,    7,   87,    4,
   56,    4,    7,    7,    7,    7,    7,    7,    7,    7,
   59,   62,   68,   78,    7,   65,    7,    7,   87,   90,
    7,    7,    7,    7,   21,   22,    4,   23,   24,   25,
   26,   56,    4,    7,    7,   92,  105,   45,  109,    7,
    7,    7,    7,    7,    7,    7,    7,    5,    6,    4,
  110,  112,  113,    7,    8,    9,   10,   11,   12,   13,
   14,  114,   46,   96,    5,    6,   94,   95,   97,   58,
    7,    8,    9,   10,   11,   12,   13,   14,    4,   45,
   59,   45,   45,   61,   64,   45,   45,   45,   45,   67,
   70,   91,    4,    5,    6,    0,   62,    0,    0,    7,
    8,    9,   10,   11,   12,   13,   14,    5,    6,    0,
    0,    0,    0,    7,    8,    9,   10,   11,   12,   13,
   14,    4,    0,    0,    0,    0,   65,   42,   44,   46,
   48,   50,    0,    0,    0,    4,    5,    6,   68,    0,
    0,    0,    7,    8,    9,   10,   11,   12,   13,   14,
    5,    6,    0,    0,    0,    0,    7,    8,    9,   10,
   11,   12,   13,   14,    4,    0,    0,    0,    0,    0,
   71,    0,    0,    0,    0,    0,    0,    0,    0,    5,
    6,    0,    0,    0,    0,    7,    8,    9,   10,   11,
   12,   13,   14,    4,    0,    0,    0,    0,    0,    0,
   73,    0,    0,    0,    0,    0,    0,    0,    5,    6,
    0,    0,    0,    0,    7,    8,    9,   10,   11,   12,
   13,   14,    4,    0,    0,    0,    0,    0,    0,    0,
   75,    0,    0,    0,    0,    0,    4,    5,    6,    0,
    0,    0,    0,    7,    8,    9,   10,   11,   12,   13,
   14,    5,    6,    0,    0,    0,    0,    7,    8,    9,
   10,   11,   12,   13,   14,   43,   45,   47,   49,   51,
   52,   53,   54,    0,    0,    0,    0,    0,    0,   43,
   45,   47,   49,   51,
};
static const YYINT yycheck[] = {                          2,
    2,   19,    2,    2,    2,    2,   80,  264,  262,   89,
  264,  265,   86,  256,  268,  269,  270,  271,  261,   22,
   23,   24,   25,   26,  104,   43,  257,   45,  285,   47,
  262,   49,  260,   51,   52,   53,   54,  262,  259,  264,
  265,  285,  263,  268,  269,  270,  271,  272,  273,  274,
  278,  279,  262,  266,  275,  276,  281,  282,  286,  262,
  265,  264,  287,  288,  289,  290,  291,  292,  293,  294,
  268,  270,  269,  259,  262,  271,  264,  265,  286,   82,
  268,  269,  270,  271,   87,   87,  266,   87,   87,   87,
   87,  265,  266,  281,  282,  277,  258,  285,  264,  287,
  288,  289,  290,  291,  292,  293,  294,  281,  282,  266,
  280,  264,  264,  287,  288,  289,  290,  291,  292,  293,
  294,  264,  285,   87,  281,  282,  283,  284,   87,   43,
  287,  288,  289,  290,  291,  292,  293,  294,  266,  262,
  268,  264,  265,   45,   47,  268,  269,  270,  271,   49,
   51,   82,  266,  281,  282,   -1,  270,   -1,   -1,  287,
  288,  289,  290,  291,  292,  293,  294,  281,  282,   -1,
   -1,   -1,   -1,  287,  288,  289,  290,  291,  292,  293,
  294,  266,   -1,   -1,   -1,   -1,  271,   22,   23,   24,
   25,   26,   -1,   -1,   -1,  266,  281,  282,  269,   -1,
   -1,   -1,  287,  288,  289,  290,  291,  292,  293,  294,
  281,  282,   -1,   -1,   -1,   -1,  287,  288,  289,  290,
  291,  292,  293,  294,  266,   -1,   -1,   -1,   -1,   -1,
  272,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,
  282,   -1,   -1,   -1,   -1,  287,  288,  289,  290,  291,
  292,  293,  294,  266,   -1,   -1,   -1,   -1,   -1,   -1,
  273,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282,
   -1,   -1,   -1,   -1,  287,  288,  289,  290,  291,  292,
  293,  294,  266,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  274,   -1,   -1,   -1,   -1,   -1,  266,  281,  282,   -1,
   -1,   -1,   -1,  287,  288,  289,  290,  291,  292,  293,
  294,  281,  282,   -1,   -1,   -1,   -1,  287,  288,  289,
  290,  291,  292,  293,  294,   22,   23,   24,   25,   26,
   27,   28,   29,   -1,   -1,   -1,   -1,   -1,   -1,   36,
   37,   38,   39,   40,
};
#define YYFINAL 3
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 294
#define YYUNDFTOKEN 334
#define YYTRANSLATE(a) ((a) > YYMAXTOKEN ? YYUNDFTOKEN : (a))
#if YYDEBUG
static const char *const yyname[] = {

"end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"T_end_br","T_end_img","T_row",
"T_end_row","T_html","T_end_html","T_end_table","T_end_cell","T_end_font",
"T_string","T_error","T_n_italic","T_n_bold","T_n_underline","T_n_overline",
"T_n_sup","T_n_sub","T_n_s","T_HR","T_hr","T_end_hr","T_VR","T_vr","T_end_vr",
"T_BR","T_br","T_IMG","T_img","T_table","T_cell","T_font","T_italic","T_bold",
"T_underline","T_overline","T_sup","T_sub","T_s",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"illegal-symbol",
};
static const char *const yyrule[] = {
"$accept : html",
"html : T_html fonttext T_end_html",
"html : T_html fonttable T_end_html",
"html : error",
"fonttext : text",
"text : text textitem",
"text : textitem",
"textitem : string",
"textitem : br",
"textitem : font text n_font",
"textitem : italic text n_italic",
"textitem : underline text n_underline",
"textitem : overline text n_overline",
"textitem : bold text n_bold",
"textitem : sup text n_sup",
"textitem : sub text n_sub",
"textitem : strike text n_strike",
"font : T_font",
"n_font : T_end_font",
"italic : T_italic",
"n_italic : T_n_italic",
"bold : T_bold",
"n_bold : T_n_bold",
"strike : T_s",
"n_strike : T_n_s",
"underline : T_underline",
"n_underline : T_n_underline",
"overline : T_overline",
"n_overline : T_n_overline",
"sup : T_sup",
"n_sup : T_n_sup",
"sub : T_sub",
"n_sub : T_n_sub",
"br : T_br T_end_br",
"br : T_BR",
"string : T_string",
"string : string T_string",
"$$1 :",
"table : opt_space T_table $$1 rows T_end_table opt_space",
"fonttable : table",
"fonttable : font table n_font",
"fonttable : italic table n_italic",
"fonttable : underline table n_underline",
"fonttable : overline table n_overline",
"fonttable : bold table n_bold",
"opt_space : string",
"opt_space :",
"rows : row",
"rows : rows row",
"rows : rows HR row",
"$$2 :",
"row : T_row $$2 cells T_end_row",
"cells : cell",
"cells : cells cell",
"cells : cells VR cell",
"$$3 :",
"cell : T_cell fonttable $$3 T_end_cell",
"$$4 :",
"cell : T_cell fonttext $$4 T_end_cell",
"$$5 :",
"cell : T_cell image $$5 T_end_cell",
"$$6 :",
"cell : T_cell $$6 T_end_cell",
"image : T_img T_end_img",
"image : T_IMG",
"HR : T_hr T_end_hr",
"HR : T_HR",
"VR : T_vr T_end_vr",
"VR : T_VR",

};
#endif

#if YYDEBUG
int      yydebug;
#endif

int      yyerrflag;
int      yychar;
YYSTYPE  yyval;
YYSTYPE  yylval;
int      yynerrs;

/* define the initial stack-sizes */
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH  YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 10000
#define YYMAXDEPTH  10000
#endif
#endif

#define YYINITSTACKSIZE 200

typedef struct {
    unsigned stacksize;
    YYINT    *s_base;
    YYINT    *s_mark;
    YYINT    *s_last;
    YYSTYPE  *l_base;
    YYSTYPE  *l_mark;
} YYSTACKDATA;
/* variables for the parser stack */
static YYSTACKDATA yystack;
#line 593 "../../lib/common/htmlparse.y"

/* parseHTML:
 * Return parsed label or NULL if failure.
 * Set warn to 0 on success; 1 for warning message; 2 if no expat.
 */
htmllabel_t*
parseHTML (char* txt, int* warn, htmlenv_t *env)
{
  unsigned char buf[SMALLBUF];
  agxbuf        str;
  htmllabel_t*  l;
  sfont_t       dfltf;

  dfltf.cfont = NULL;
  dfltf.pfont = NULL;
  HTMLstate.fontstack = &dfltf;
  HTMLstate.tblstack = 0;
  HTMLstate.lbl = 0;
  HTMLstate.gvc = GD_gvc(env->g);
  HTMLstate.fitemList = dtopen(&fstrDisc, Dtqueue);
  HTMLstate.fspanList = dtopen(&fspanDisc, Dtqueue);

  agxbinit (&str, SMALLBUF, buf);
  HTMLstate.str = &str;
  
  if (initHTMLlexer (txt, &str, env)) {/* failed: no libexpat - give up */
    *warn = 2;
    l = NULL;
  }
  else {
    yyparse();
    *warn = clearHTMLlexer ();
    l = HTMLstate.lbl;
  }

  dtclose (HTMLstate.fitemList);
  dtclose (HTMLstate.fspanList);
  
  HTMLstate.fitemList = NULL;
  HTMLstate.fspanList = NULL;
  HTMLstate.fontstack = NULL;
  
  agxbfree (&str);

  return l;
}

#line 842 "y.tab.c"

#if YYDEBUG
#include <stdio.h>	/* needed for printf */
#endif

#include <stdlib.h>	/* needed for malloc, etc */
#include <string.h>	/* needed for memset */

/* allocate initial stack or double stack size, up to YYMAXDEPTH */
static int yygrowstack(YYSTACKDATA *data)
{
    int i;
    unsigned newsize;
    YYINT *newss;
    YYSTYPE *newvs;

    if ((newsize = data->stacksize) == 0)
        newsize = YYINITSTACKSIZE;
    else if (newsize >= YYMAXDEPTH)
        return YYENOMEM;
    else if ((newsize *= 2) > YYMAXDEPTH)
        newsize = YYMAXDEPTH;

    i = (int) (data->s_mark - data->s_base);
    newss = (YYINT *)realloc(data->s_base, newsize * sizeof(*newss));
    if (newss == 0)
        return YYENOMEM;

    data->s_base = newss;
    data->s_mark = newss + i;

    newvs = (YYSTYPE *)realloc(data->l_base, newsize * sizeof(*newvs));
    if (newvs == 0)
        return YYENOMEM;

    data->l_base = newvs;
    data->l_mark = newvs + i;

    data->stacksize = newsize;
    data->s_last = data->s_base + newsize - 1;
    return 0;
}

#if YYPURE || defined(YY_NO_LEAKS)
static void yyfreestack(YYSTACKDATA *data)
{
    free(data->s_base);
    free(data->l_base);
    memset(data, 0, sizeof(*data));
}
#else
#define yyfreestack(data) /* nothing */
#endif

#define YYABORT  goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR  goto yyerrlab

int
YYPARSE_DECL()
{
    int yym, yyn, yystate;
#if YYDEBUG
    const char *yys;

    if ((yys = getenv("YYDEBUG")) != 0)
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif

    yym = 0;
    yyn = 0;
    yynerrs = 0;
    yyerrflag = 0;
    yychar = YYEMPTY;
    yystate = 0;

#if YYPURE
    memset(&yystack, 0, sizeof(yystack));
#endif

    if (yystack.s_base == NULL && yygrowstack(&yystack) == YYENOMEM) goto yyoverflow;
    yystack.s_mark = yystack.s_base;
    yystack.l_mark = yystack.l_base;
    yystate = 0;
    *yystack.s_mark = 0;

yyloop:
    if ((yyn = yydefred[yystate]) != 0) goto yyreduce;
    if (yychar < 0)
    {
        yychar = YYLEX;
        if (yychar < 0) yychar = YYEOF;
#if YYDEBUG
        if (yydebug)
        {
            if ((yys = yyname[YYTRANSLATE(yychar)]) == NULL) yys = yyname[YYUNDFTOKEN];
            printf("%sdebug: state %d, reading %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
    }
    if (((yyn = yysindex[yystate]) != 0) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == (YYINT) yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: state %d, shifting to state %d\n",
                    YYPREFIX, yystate, yytable[yyn]);
#endif
        if (yystack.s_mark >= yystack.s_last && yygrowstack(&yystack) == YYENOMEM) goto yyoverflow;
        yystate = yytable[yyn];
        *++yystack.s_mark = yytable[yyn];
        *++yystack.l_mark = yylval;
        yychar = YYEMPTY;
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if (((yyn = yyrindex[yystate]) != 0) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == (YYINT) yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag != 0) goto yyinrecovery;

    YYERROR_CALL("syntax error");

    goto yyerrlab; /* redundant goto avoids 'unused label' warning */
yyerrlab:
    ++yynerrs;

yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if (((yyn = yysindex[*yystack.s_mark]) != 0) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == (YYINT) YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: state %d, error recovery shifting\
 to state %d\n", YYPREFIX, *yystack.s_mark, yytable[yyn]);
#endif
                if (yystack.s_mark >= yystack.s_last && yygrowstack(&yystack) == YYENOMEM) goto yyoverflow;
                yystate = yytable[yyn];
                *++yystack.s_mark = yytable[yyn];
                *++yystack.l_mark = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: error recovery discarding state %d\n",
                            YYPREFIX, *yystack.s_mark);
#endif
                if (yystack.s_mark <= yystack.s_base) goto yyabort;
                --yystack.s_mark;
                --yystack.l_mark;
            }
        }
    }
    else
    {
        if (yychar == YYEOF) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            if ((yys = yyname[YYTRANSLATE(yychar)]) == NULL) yys = yyname[YYUNDFTOKEN];
            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
        yychar = YYEMPTY;
        goto yyloop;
    }

yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
                YYPREFIX, yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    if (yym > 0)
        yyval = yystack.l_mark[1-yym];
    else
        memset(&yyval, 0, sizeof yyval);

    switch (yyn)
    {
case 1:
#line 447 "../../lib/common/htmlparse.y"
	{ HTMLstate.lbl = mkLabel(yystack.l_mark[-1].txt,HTML_TEXT); }
break;
case 2:
#line 448 "../../lib/common/htmlparse.y"
	{ HTMLstate.lbl = mkLabel(yystack.l_mark[-1].tbl,HTML_TBL); }
break;
case 3:
#line 449 "../../lib/common/htmlparse.y"
	{ cleanup(); YYABORT; }
break;
case 4:
#line 452 "../../lib/common/htmlparse.y"
	{ yyval.txt = mkText(); }
break;
case 7:
#line 459 "../../lib/common/htmlparse.y"
	{ appendFItemList(HTMLstate.str);}
break;
case 8:
#line 460 "../../lib/common/htmlparse.y"
	{appendFLineList(yystack.l_mark[0].i);}
break;
case 17:
#line 471 "../../lib/common/htmlparse.y"
	{ pushFont (yystack.l_mark[0].font); }
break;
case 18:
#line 474 "../../lib/common/htmlparse.y"
	{ popFont (); }
break;
case 19:
#line 477 "../../lib/common/htmlparse.y"
	{pushFont(yystack.l_mark[0].font);}
break;
case 20:
#line 480 "../../lib/common/htmlparse.y"
	{popFont();}
break;
case 21:
#line 483 "../../lib/common/htmlparse.y"
	{pushFont(yystack.l_mark[0].font);}
break;
case 22:
#line 486 "../../lib/common/htmlparse.y"
	{popFont();}
break;
case 23:
#line 489 "../../lib/common/htmlparse.y"
	{pushFont(yystack.l_mark[0].font);}
break;
case 24:
#line 492 "../../lib/common/htmlparse.y"
	{popFont();}
break;
case 25:
#line 495 "../../lib/common/htmlparse.y"
	{pushFont(yystack.l_mark[0].font);}
break;
case 26:
#line 498 "../../lib/common/htmlparse.y"
	{popFont();}
break;
case 27:
#line 501 "../../lib/common/htmlparse.y"
	{pushFont(yystack.l_mark[0].font);}
break;
case 28:
#line 504 "../../lib/common/htmlparse.y"
	{popFont();}
break;
case 29:
#line 507 "../../lib/common/htmlparse.y"
	{pushFont(yystack.l_mark[0].font);}
break;
case 30:
#line 510 "../../lib/common/htmlparse.y"
	{popFont();}
break;
case 31:
#line 513 "../../lib/common/htmlparse.y"
	{pushFont(yystack.l_mark[0].font);}
break;
case 32:
#line 516 "../../lib/common/htmlparse.y"
	{popFont();}
break;
case 33:
#line 519 "../../lib/common/htmlparse.y"
	{ yyval.i = yystack.l_mark[-1].i; }
break;
case 34:
#line 520 "../../lib/common/htmlparse.y"
	{ yyval.i = yystack.l_mark[0].i; }
break;
case 37:
#line 527 "../../lib/common/htmlparse.y"
	{ 
          if (nonSpace(agxbuse(HTMLstate.str))) {
            yyerror ("Syntax error: non-space string used before <TABLE>");
            cleanup(); YYABORT;
          }
          yystack.l_mark[0].tbl->u.p.prev = HTMLstate.tblstack;
          yystack.l_mark[0].tbl->u.p.rows = dtopen(&rowDisc, Dtqueue);
          HTMLstate.tblstack = yystack.l_mark[0].tbl;
          yystack.l_mark[0].tbl->font = HTMLstate.fontstack->cfont;
          yyval.tbl = yystack.l_mark[0].tbl;
        }
break;
case 38:
#line 538 "../../lib/common/htmlparse.y"
	{
          if (nonSpace(agxbuse(HTMLstate.str))) {
            yyerror ("Syntax error: non-space string used after </TABLE>");
            cleanup(); YYABORT;
          }
          yyval.tbl = HTMLstate.tblstack;
          HTMLstate.tblstack = HTMLstate.tblstack->u.p.prev;
        }
break;
case 39:
#line 548 "../../lib/common/htmlparse.y"
	{ yyval.tbl = yystack.l_mark[0].tbl; }
break;
case 40:
#line 549 "../../lib/common/htmlparse.y"
	{ yyval.tbl=yystack.l_mark[-1].tbl; }
break;
case 41:
#line 550 "../../lib/common/htmlparse.y"
	{ yyval.tbl=yystack.l_mark[-1].tbl; }
break;
case 42:
#line 551 "../../lib/common/htmlparse.y"
	{ yyval.tbl=yystack.l_mark[-1].tbl; }
break;
case 43:
#line 552 "../../lib/common/htmlparse.y"
	{ yyval.tbl=yystack.l_mark[-1].tbl; }
break;
case 44:
#line 553 "../../lib/common/htmlparse.y"
	{ yyval.tbl=yystack.l_mark[-1].tbl; }
break;
case 47:
#line 560 "../../lib/common/htmlparse.y"
	{ yyval.p = yystack.l_mark[0].p; }
break;
case 48:
#line 561 "../../lib/common/htmlparse.y"
	{ yyval.p = yystack.l_mark[0].p; }
break;
case 49:
#line 562 "../../lib/common/htmlparse.y"
	{ yystack.l_mark[-2].p->ruled = 1; yyval.p = yystack.l_mark[0].p; }
break;
case 50:
#line 565 "../../lib/common/htmlparse.y"
	{ addRow (); }
break;
case 51:
#line 565 "../../lib/common/htmlparse.y"
	{ yyval.p = lastRow(); }
break;
case 52:
#line 568 "../../lib/common/htmlparse.y"
	{ yyval.cell = yystack.l_mark[0].cell; }
break;
case 53:
#line 569 "../../lib/common/htmlparse.y"
	{ yyval.cell = yystack.l_mark[0].cell; }
break;
case 54:
#line 570 "../../lib/common/htmlparse.y"
	{ yystack.l_mark[-2].cell->ruled |= HTML_VRULE; yyval.cell = yystack.l_mark[0].cell; }
break;
case 55:
#line 573 "../../lib/common/htmlparse.y"
	{ setCell(yystack.l_mark[-1].cell,yystack.l_mark[0].tbl,HTML_TBL); }
break;
case 56:
#line 573 "../../lib/common/htmlparse.y"
	{ yyval.cell = yystack.l_mark[-3].cell; }
break;
case 57:
#line 574 "../../lib/common/htmlparse.y"
	{ setCell(yystack.l_mark[-1].cell,yystack.l_mark[0].txt,HTML_TEXT); }
break;
case 58:
#line 574 "../../lib/common/htmlparse.y"
	{ yyval.cell = yystack.l_mark[-3].cell; }
break;
case 59:
#line 575 "../../lib/common/htmlparse.y"
	{ setCell(yystack.l_mark[-1].cell,yystack.l_mark[0].img,HTML_IMAGE); }
break;
case 60:
#line 575 "../../lib/common/htmlparse.y"
	{ yyval.cell = yystack.l_mark[-3].cell; }
break;
case 61:
#line 576 "../../lib/common/htmlparse.y"
	{ setCell(yystack.l_mark[0].cell,mkText(),HTML_TEXT); }
break;
case 62:
#line 576 "../../lib/common/htmlparse.y"
	{ yyval.cell = yystack.l_mark[-2].cell; }
break;
case 63:
#line 579 "../../lib/common/htmlparse.y"
	{ yyval.img = yystack.l_mark[-1].img; }
break;
case 64:
#line 580 "../../lib/common/htmlparse.y"
	{ yyval.img = yystack.l_mark[0].img; }
break;
#line 1258 "y.tab.c"
    }
    yystack.s_mark -= yym;
    yystate = *yystack.s_mark;
    yystack.l_mark -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: after reduction, shifting from state 0 to\
 state %d\n", YYPREFIX, YYFINAL);
#endif
        yystate = YYFINAL;
        *++yystack.s_mark = YYFINAL;
        *++yystack.l_mark = yyval;
        if (yychar < 0)
        {
            yychar = YYLEX;
            if (yychar < 0) yychar = YYEOF;
#if YYDEBUG
            if (yydebug)
            {
                if ((yys = yyname[YYTRANSLATE(yychar)]) == NULL) yys = yyname[YYUNDFTOKEN];
                printf("%sdebug: state %d, reading %d (%s)\n",
                        YYPREFIX, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == YYEOF) goto yyaccept;
        goto yyloop;
    }
    if (((yyn = yygindex[yym]) != 0) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == (YYINT) yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: after reduction, shifting from state %d \
to state %d\n", YYPREFIX, *yystack.s_mark, yystate);
#endif
    if (yystack.s_mark >= yystack.s_last && yygrowstack(&yystack) == YYENOMEM) goto yyoverflow;
    *++yystack.s_mark = (YYINT) yystate;
    *++yystack.l_mark = yyval;
    goto yyloop;

yyoverflow:
    YYERROR_CALL("yacc stack overflow");

yyabort:
    yyfreestack(&yystack);
    return (1);

yyaccept:
    yyfreestack(&yystack);
    return (0);
}
